pref("general.useragent.security",       "U");

pref("security.enable_ssl2",             true);
pref("security.enable_ssl3",             true);
pref("security.enable_tls",		 true);

pref("security.ssl2.rc4_128", true);
pref("security.ssl2.rc2_128", true);
pref("security.ssl2.des_ede3_192", true);
pref("security.ssl2.des_64", true);
pref("security.ssl2.rc4_40", true);
pref("security.ssl2.rc2_40", true);
pref("security.ssl3.rsa_rc4_128_md5", true);
pref("security.ssl3.rsa_rc4_128_sha", true);
pref("security.ssl3.rsa_fips_des_ede3_sha", true);
pref("security.ssl3.rsa_des_ede3_sha", true);
pref("security.ssl3.rsa_fips_des_sha", true);
pref("security.ssl3.rsa_des_sha", true);
pref("security.ssl3.rsa_1024_rc4_56_sha", true);
pref("security.ssl3.rsa_1024_des_cbc_sha", true);
pref("security.ssl3.rsa_rc4_40_md5", true);
pref("security.ssl3.rsa_rc2_40_md5", true);
pref("security.ssl3.rsa_aes_256_sha", true);
pref("security.ssl3.rsa_aes_128_sha", true);
pref("security.ssl3.rsa_null_sha", false);
pref("security.ssl3.rsa_null_md5", false);

pref("security.default_personal_cert",   "Select Automatically");
pref("security.ask_for_password",        0);
pref("security.password_lifetime",       30);
pref("security.warn_entering_secure",    true);
pref("security.warn_entering_weak",      true);
pref("security.warn_leaving_secure",     true);
pref("security.warn_viewing_mixed",      true);
pref("security.warn_submit_insecure",    true);

pref("security.OCSP.enabled",		0);

pref("security.ui.enable",    true);


